class Adipico{
  
  // propriedades da classe
  float xpos, ypos; // posição do átomo de carbono que fica mais à direita
  float offset = comprimento/30; // define a separação entre as ligações da dupla-ligação
  
  // váriáveis de animação
  
  float w = 0; // define a abertura das ligações no átomo de carbono à DIREITA (se seu valor é '1',
               // não desenha-se um dos átomos de oxigênio e um dos de hidrogênio)
               
  float u = 0; // define a abertura das ligações no átomo de carbono à ESQUERDA(se seu valor é '1', 
               // não desenhamos um dos átomos de oxigênio e um dos de hidrogênio)
  
  // CONSTRUTORA
  Adipico(float x, float y){
    xpos = x;
    ypos = y;
  }
  
  // função que desenha a molécula de monômero
  void display(){
    
    offset = comprimento/30; // atualizamos o valor do offset, caso 'comprimento' tenha sido mudado
    
    pushMatrix();
    translate(xpos, ypos); // origem: posição do átomo de carbono mais à direita
    stroke(0);
    line(0, 0, -comprimento/4, 0);
    // desenhamos os átomos ligados ao carbono mais à esquerda
    pushMatrix();
    rotate( - PI/3 - w*PI/6); // giramos para desenhar a ligação dupla
    line(0, offset, comprimento/2, offset);        // ligação
    line(0, - offset, comprimento/2, - offset);    //  dupla
    noStroke();
    fill(#FC928F); // ROSA
    ellipse(comprimento/2, 0, 1.1*comprimento/5, 1.1*comprimento/5); // desenha o átomo de oxigênio
    popMatrix();
    pushMatrix();
    rotate(PI/3 - w*PI/3); // giramos para desenhar o outro átomo de oxigênio, ligado a um hidrogênio
    stroke(0);
    line(0, 0, comprimento/4, 0);
    
    if(etapa == 0){ // só desenhamos estes átomos se a etapa for 0 
      pushMatrix();
      translate(comprimento/2, 0); // origem: átomo de oxigênio ligado simplesmente ao carbono mais à direita
      rotate(w*PI);
      line(0, 0, -comprimento/4, 0);
      rotate(- PI/3);
      line(0, 0, comprimento/2, 0);
      noStroke(); // para não haver contorno
      fill(122); // CINZA
      ellipse(comprimento/2, 0, comprimento/5, comprimento/5); // desenhamos o átomo de hidrogênio
      fill(#FC928F); // ROSA
      ellipse(0, 0, 1.1*comprimento/5, 1.1*comprimento/5); // desenhamos o segundo átomo de oxigênio
      popMatrix(); // voltamos com a origem no átomo de carbono mais à direita
    }
    
    noStroke();
    fill(#91D6F7); // AZUL
    ellipse(0, 0, 1.3*comprimento/5, 1.3*comprimento/5); // desenhamos o átomo de carbono mais à direita
    popMatrix();
    
    // para desenhar os carbonos internos à molécula, com seus hidrogênios ligados:    
    for(int i = 1; i <= 4; i++){
      pushMatrix();
      translate(- i*comprimento/2, 0); // mudamos a origem para a posição do carbono a ser desenhado
      stroke(0);
      line(0, 0, 0, 5*comprimento/12);
      line(0, 0, 0, - 5*comprimento/12);
      line(0, 0, comprimento/4, 0);
      line(0, 0, - comprimento/4, 0);
      noStroke();
      fill(122); // CINZA
      ellipse(0, 5*comprimento/12, comprimento/5, comprimento/5);         // desenhamos os átomos
      ellipse(0, - 5*comprimento/12, comprimento/5, comprimento/5);       // de hidrogênio
      fill(#91D6F7); // AZUL
      ellipse(0, 0, 1.3*(comprimento/5), 1.3*(comprimento/5)); // desenhamos o átomo de carbono
      popMatrix();
    }
    
    // então fazemos o mesmo que fizemos com a extremidade esquerda na outra extremidade
    // espelhando as ligações para adaptarmos o desenho à simetria
    pushMatrix();
    translate(- 5*comprimento/2, 0); // origem: átomo de carbono mais à esquerda
    stroke(0);
    line(0, 0, comprimento/4, 0);
    pushMatrix();
    rotate( - 2*PI/3 + u*PI/6); // giramos para desenhar a ligação dupla
    line(0, offset, comprimento/2, offset);                // ligação
    line(0, - offset, comprimento/2, - offset);            // dupla
    noStroke();
    fill(#FC928F); // ROSA
    ellipse(comprimento/2, 0, 1.1*comprimento/5, 1.1*comprimento/5); // desenhamos o átomo de oxigênio com ligação dupla
    popMatrix();
    pushMatrix();
    rotate(2*PI/3 + u*PI/3); // giramos para desenhar a ligação simples com o outro oxigênio
    stroke(0);
    line(0, 0, comprimento/4, 0);
    
    // dependendo do valor assumido por 'u', mostramos ou não o átomo de oxigênio ligado ao hidrogênio (e ao carbono à esquerda)
    if(u != 1){ 
      pushMatrix();
      translate(comprimento/2, 0);
      line(0, 0, -comprimento/4, 0);
      rotate(PI/3); // giramos para desenhar o hidrogênio ligado
      line(0, 0, 5*comprimento/12, 0);
      noStroke();
      fill(122); // CINZA
      ellipse(5*comprimento/12, 0, comprimento/5, comprimento/5); // desenhamos o hidrogênio
      fill(#FC928F); // VERMELHO
      ellipse(0, 0, 1.1*comprimento/5, 1.1*comprimento/5); // desenhamos o oxigênio
      popMatrix();
    }    
    
    noStroke();
    fill(#91D6F7); // AZUL
    ellipse(0, 0, 1.3*comprimento/5, 1.3*comprimento/5); // desenhamos o último carbono
    popMatrix();
    popMatrix();
    popMatrix();
  }
  
  // função que faz a 'abertura' das ligações do carbono mais à direta, ao mudar o valor de 'w'
  void abre(float v){
    if(w < 1)
      w += v;
    else
      w = 1;
  }
  
  // função que faz a molécula se mover no plano
  void anda(float vx, float vy){
    xpos += vx;
    ypos += vy;
  }

  // define uma nova posição
  void setPosition(float x, float y){
    xpos = x;
    ypos = y;
  }
}
